#include "threatcheck.as"
#include "lugaru_campaign.as"

void SetParameters() {
	params.AddString("next_level", "");
    params.AddString("music", "lugaru_ambient_grass");
}

void Init() {
    SavedLevel @savedLevel = save_file.GetSavedLevel("lugaru_campaign");
    savedLevel.SetValue("current_level",GetCurrLevel());
    save_file.WriteInPlace();
    AddMusic("Data/Music/lugaru_new.xml");
    level.ReceiveLevelEvents(hotspot.GetID());
}

void Dispose() {
    level.StopReceivingLevelEvents(hotspot.GetID());
}

void CreateSecondCharacter() {
    int firstId = -1;
    int secondId = -1;

    int num = GetNumCharacters();
    for(int i=0; i<num; ++i){
        MovementObject@ char = ReadCharacter(i);
        if(char.controlled){
            if (firstId == -1) {
                firstId = char.GetID();
            } else {
                secondId = char.GetID();
                break;
            }
        }
    }

    if (firstId != -1 && secondId == -1) {
        Print("Creating second character");
        Object@ firstObject = ReadObjectFromID(firstId);
        MovementObject@ created = ReadCharacterID(DuplicateObject(firstObject));
        created.Execute("this_mo.RecreateRiggedObject(\"Data/Characters/raider_rabbit.xml\");");

        second_char_id = created.GetID();
    }
}

float blackout_amount = 0.0;
float ko_time = -1.0;
float win_time = -1.0;
bool sent_level_complete_message = false;
int second_char_id = -1;
bool in_dialogue = false;

void Update() {
    if (in_dialogue) {
        level.Execute("if (dialogue.index == 0) { level.SendMessage(\"dialogue_finished\"); }");
    }

    if (second_char_id == -1 && !in_dialogue) {
        Print("Trying to create a second character\n");
        CreateSecondCharacter();
    }

    int num = GetNumCharacters();
    bool anyone_in_combat = false;
    bool anyone_knocked_out = false;
    bool player_exists = false;
    bool attack_pressed_by_anyone = false;

    for(int i=0; i<num; ++i){
        MovementObject@ char = ReadCharacter(i);

        if(char.controlled){
            player_exists = true;

            if (!anyone_in_combat) {
                anyone_in_combat = char.QueryIntFunction("int CombatSong()") == 1 && char.GetIntVar("knocked_out") == _awake;
            }

            if (!anyone_knocked_out) {
                anyone_knocked_out = char.GetIntVar("knocked_out") != _awake;
            }

            if (!attack_pressed_by_anyone) {
                attack_pressed_by_anyone = GetInputPressed(char.controller_id, "attack");
            }
        }
    }

    if(anyone_in_combat){
        PlaySong("lugaru_combat");
    } else if(params.HasParam("music")){
        PlaySong(params.GetString("music"));
    }

	if(player_exists){
		if(anyone_knocked_out){
			if(ko_time == -1.0f){
				ko_time = the_time;
			}
			if(ko_time < the_time - 1.0){
				if(attack_pressed_by_anyone || ko_time < the_time - 5.0){
	            	level.SendMessage("reset"); 				
                    level.SendMessage("skip_dialogue");                 
				}
			}
		} else {
			ko_time = -1.0f;
		}
	} else {
        ko_time = -1.0f;
    }
	if(ThreatsRemaining() == 0 && ThreatsPossible() != 0 && ko_time == -1.0){
		if(win_time == -1.0f){
			win_time = the_time;
		}
		if(win_time < the_time - 5.0 && !sent_level_complete_message){
			string path = params.GetString("next_level");
			if(path != ""){
                FinishedLugaruCampaignLevel(GetCurrLevel());
	            level.SendMessage("loadlevel \""+path+"\"");		
                sent_level_complete_message = true;
	        }
	    }
	} else {
        win_time = -1.0;
    }
}

void PreDraw(float curr_game_time) {
    camera.SetTint(camera.GetTint() * (1.0 - blackout_amount));
}

void ReceiveMessage(string msg) {
    TokenIterator token_iter;
    token_iter.Init();

    if(!token_iter.FindNextToken(msg)){
        return;
    }

    string msg_start = token_iter.GetToken(msg);

    if (msg_start == "level_event" && token_iter.FindNextToken(msg)) {
        string ev = token_iter.GetToken(msg);

        if(ev == "start_dialogue"){
            in_dialogue = true;

            if (second_char_id != -1 && ObjectExists(second_char_id)) {
                Print("Deleting second char " + second_char_id + "\n");
                DeleteObjectID(second_char_id);
                second_char_id = -1;
            }
        }

        if(ev == "dialogue_finished"){
            Print("Dialogue finished" + "\n");
            in_dialogue = false;
        }
    }
}

void Draw(){
    if(EditorModeActive()){
        Object@ obj = ReadObjectFromID(hotspot.GetID());
        DebugDrawBillboard("Data/Textures/ui/lugaru_icns_256x256.png",
                           obj.GetTranslation(),
                           obj.GetScale()[1]*2.0,
                           vec4(vec3(0.5), 1.0),
                           _delete_on_draw);
    }
}